//
//  M A G N E T O
// ===============
// Datei         : Display.c
// Funktion      : Alle Routinen zur Display Darstellung.
//

#include <avr/io.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include <stdlib.h>
#include "Display.h"
#include "Schriften.h"
#include "Bildschirm.h"




uint8_t Display_Shadow[128][8];					// Im RAM 1024 Bytes fr das Abbild vom Display reservieren.
												// Organisiert in 128 Bytes in horizontalen Spalten
												// und 8 Zeilen.
extern const unsigned short Zeichen[][6];




void Display_Shadow_set_Pixel (uint8_t x, uint8_t y, uint8_t z) // x max = 128; y max = 64
{
	uint8_t Zeile = 0;
	
	if (y <= 8)
	{
		Zeile = 1;
	}
	else if (y >  8 && y <= 16)
	{
		Zeile = 2;
		y = y-8;
	}
	else if (y > 16 && y <= 24)
	{
		Zeile = 3;
		y = y-16;
	}
	else if (y > 24 && y <= 32)
	{
		Zeile = 4;
		y = y-24;
	}
	else if (y > 32 && y <= 40)
	{
		Zeile = 5;
		y = y-32;
	}
	else if (y > 40 && y <= 48)
	{
		Zeile = 6;
		y = y-40;
	}
	else if (y > 48 && y <= 56)
	{
		Zeile = 7;
		y = y-48;
	}
	else if (y > 56 && y <= 64)
	{
		Zeile = 8;
		y = y-56;
	}
	
	if (z==1) Display_Shadow[x-1][Zeile-1]  |=  (1<<(y-1));
	if (z==0) Display_Shadow[x-1][Zeile-1]  &=  ~(1<<(y-1));
}

//
// Display Shadow RAM lschen, alles auf 0x00 setzen.
//
void Display_Shadow_loeschen(void)
{
	for (int a=0; a<8; a++)
	{
		for (int b=0; b<128; b++)
		{
			Display_Shadow[b][a] = 0x00;
		}
	}
}

//
// Display Shadow RAM auf das Display schreiben.
//
void Display_aktualisieren(void)
{
	int a, b, c, d;
	
	for (a=0; a < 8; a++) // 8 Zeilen (vertikal)
	{
		SPI_Display_schreiben(0xB0 + a);
		d=0;
		
		for (b=0; b<8; b++)										// 8 x 16 Bytes = 128 Bytes Zeichen (horizontal).
		{
			if (b>0) d=d+16;									// jeweils 16 Bits als Block dazuzhlen.
			if (d>127) d=0;										// In jeder Zeile von vorne zu zhlen beginnen.
			
			for (c=0; c < 16; c++)
			{
				SPI_Display_schreiben(0x10 + b);					// Nach 16 Bits diese Adresse um eins erhhen (8 Blcke),
				SPI_Display_schreiben(0x00 + c);					// und diese Adresse jeweils 16 Bits hintereinander schreiben.
				
				PORTB |= (1<<A0);								// A0 fr Daten schreiben setzen.
				SPI_Display_schreiben(Display_Shadow[c+d][a]);		// Daten zum Display.
				PORTB &= ~(1 << A0);							// A0 fr Befehle wieder rcksetzen.
			}
		}
	}
	
}

//
// Richtung: 0 = horizontal.
//           1 = vertikal.
// setzen    0 = Bit lschen.
//			 1 = Bit setzen.
//
void Display_Linie(uint8_t x, uint8_t y, int16_t Laenge, uint8_t Richtung, uint8_t setzen)
{
	if (Richtung == 0)
	{
		if (Laenge > 0)
		for (; Laenge > 0; Laenge--, x++)
		{
			if (setzen == 0) Display_Shadow_set_Pixel(x, y, 0);
			if (setzen == 1) Display_Shadow_set_Pixel(x, y, 1);
		}
		else
		for (; Laenge < 0; Laenge++, x--)
		{
			if (setzen == 0) Display_Shadow_set_Pixel(x, y, 0);
			if (setzen == 1) Display_Shadow_set_Pixel(x, y, 1);
		}
	}
	else
	{
		if (Laenge > 0)
		for (; Laenge > 0; Laenge--, y++)
		{
			if (setzen == 0) Display_Shadow_set_Pixel(x, y, 0);
			if (setzen == 1) Display_Shadow_set_Pixel(x, y, 1);
		}
		else
		for (; Laenge < 0; Laenge++, y--)
		{
			if (setzen == 0) Display_Shadow_set_Pixel(x, y, 0);
			if (setzen == 1) Display_Shadow_set_Pixel(x, y, 1);
		}
	}
}

//
// Ein Zeichen auf das Display schreiben
// fr 6x8 Font.
//
// x, y Position.
// Zeichen als ASCII.
// setzen 0 = lschen.
//        1 = schreiben.
//
void Display_Zeichen(uint8_t x, uint8_t y, char Ausgabe, uint8_t setzen)
{
	//int a = 0;
	
	for (int i = 0; i <	6; i++)
	{
		Display_Shadow[x+i][y-1] = Zeichen[Ausgabe - ' '][i];
	}
}

void Display_String(uint8_t x, uint8_t y, char *String)
{
	
}

void Display_Zeichen_Pixel(uint8_t x, uint8_t y, char Ausgabe, uint8_t setzen)
{
	char Ausgabe_temp_oben;
	char Ausgabe_temp_unten;
	
	if (y==1)
	{
		for (int i = 0; i <	6; i++)
		{
			Display_Shadow[x+i][y-1] = Zeichen[Ausgabe - ' '][i];
		}
	}
	
	else if (y >= 2 && y <= 9)
	{
		int z = 1; // Zeilen Pixel Zhler
		
		for (int i = 0; i < 6; i++)
		{
			Ausgabe_temp_oben = Zeichen[Ausgabe - ' '][i];
			Ausgabe_temp_oben = Ausgabe_temp_oben << (y-z);
			if (setzen == 0) Display_Shadow[x+i][0] |= Ausgabe_temp_oben;
			if (setzen == 1) Display_Shadow[x+i][0] &= ~Ausgabe_temp_oben;
			if (setzen == 2) Display_Shadow[x+i][0] = Ausgabe_temp_oben ^ 0xFF;
		}
		for (int i = 0; i < 6; i++)
		{
			Ausgabe_temp_unten = Zeichen[Ausgabe - ' '][i];
			Ausgabe_temp_unten = Ausgabe_temp_unten >> (8 - (y-z));
			if (setzen == 0) Display_Shadow[x+i][1] |= Ausgabe_temp_unten;
			if (setzen == 1) Display_Shadow[x+i][1] &= ~Ausgabe_temp_unten;
			if (setzen == 2) Display_Shadow[x+i][1] = Ausgabe_temp_unten ^ 0xFF;
		}
	}

	else if (y >= 10 && y <= 17)
	{
		int z = 9; // Zeilen Pixel Zhler
		
		for (int i = 0; i < 6; i++)
		{
			Ausgabe_temp_oben = Zeichen[Ausgabe - ' '][i];
			Ausgabe_temp_oben = Ausgabe_temp_oben << (y-z);
			if (setzen == 0) Display_Shadow[x+i][1] |= Ausgabe_temp_oben;
			if (setzen == 1) Display_Shadow[x+i][1] &= ~Ausgabe_temp_oben;
			if (setzen == 2) Display_Shadow[x+i][1] = Ausgabe_temp_oben ^ 0xFF;
		}
		for (int i = 0; i < 6; i++)
		{
			Ausgabe_temp_unten = Zeichen[Ausgabe - ' '][i];
			Ausgabe_temp_unten = Ausgabe_temp_unten >> (8 - (y-z));
			if (setzen == 0) Display_Shadow[x+i][2] |= Ausgabe_temp_unten;
			if (setzen == 1) Display_Shadow[x+i][2] &= ~Ausgabe_temp_unten;
			if (setzen == 2) Display_Shadow[x+i][2] = Ausgabe_temp_unten ^ 0xFF;
		}
	}

	else if (y >= 18 && y <= 25)
	{
		int z = 17; // Zeilen Pixel Zhler
		
		for (int i = 0; i < 6; i++)
		{
			Ausgabe_temp_oben = Zeichen[Ausgabe - ' '][i];
			Ausgabe_temp_oben = Ausgabe_temp_oben << (y-z);
			if (setzen == 0) Display_Shadow[x+i][2] |= Ausgabe_temp_oben;
			if (setzen == 1) Display_Shadow[x+i][2] &= ~Ausgabe_temp_oben;
			if (setzen == 2) Display_Shadow[x+i][2] = Ausgabe_temp_oben ^ 0xFF;
		}
		for (int i = 0; i < 6; i++)
		{
			Ausgabe_temp_unten = Zeichen[Ausgabe - ' '][i];
			Ausgabe_temp_unten = Ausgabe_temp_unten >> (8 - (y-z));
			if (setzen == 0) Display_Shadow[x+i][3] |= Ausgabe_temp_unten;
			if (setzen == 1) Display_Shadow[x+i][3] &= ~Ausgabe_temp_unten;
			if (setzen == 2) Display_Shadow[x+i][3] = Ausgabe_temp_unten ^ 0xFF;
		}
	}

	else if (y >= 26 && y <= 33)
	{
		int z = 25; // Zeilen Pixel Zhler
		
		for (int i = 0; i < 6; i++)
		{
			Ausgabe_temp_oben = Zeichen[Ausgabe - ' '][i];
			Ausgabe_temp_oben = Ausgabe_temp_oben << (y-z);
			if (setzen == 0) Display_Shadow[x+i][3] |= Ausgabe_temp_oben;
			if (setzen == 1) Display_Shadow[x+i][3] &= ~Ausgabe_temp_oben;
			if (setzen == 2) Display_Shadow[x+i][3] = Ausgabe_temp_oben ^ 0xFF;
		}
		for (int i = 0; i < 6; i++)
		{
			Ausgabe_temp_unten = Zeichen[Ausgabe - ' '][i];
			Ausgabe_temp_unten = Ausgabe_temp_unten >> (8 - (y-z));
			if (setzen == 0) Display_Shadow[x+i][4] |= Ausgabe_temp_unten;
			if (setzen == 1) Display_Shadow[x+i][4] &= ~Ausgabe_temp_unten;
			if (setzen == 2) Display_Shadow[x+i][4] = Ausgabe_temp_unten ^ 0xFF;
		}
	}

	else if (y >= 34 && y <= 41)
	{
		int z = 33; // Zeilen Pixel Zhler
		
		for (int i = 0; i < 6; i++)
		{
			Ausgabe_temp_oben = Zeichen[Ausgabe - ' '][i];
			Ausgabe_temp_oben = Ausgabe_temp_oben << (y-z);
			if (setzen == 0) Display_Shadow[x+i][4] |= Ausgabe_temp_oben;
			if (setzen == 1) Display_Shadow[x+i][4] &= ~Ausgabe_temp_oben;
			if (setzen == 2) Display_Shadow[x+i][4] = Ausgabe_temp_oben ^ 0xFF;
		}
		for (int i = 0; i < 6; i++)
		{
			Ausgabe_temp_unten = Zeichen[Ausgabe - ' '][i];
			Ausgabe_temp_unten = Ausgabe_temp_unten >> (8 - (y-z));
			if (setzen == 0) Display_Shadow[x+i][5] |= Ausgabe_temp_unten;
			if (setzen == 1) Display_Shadow[x+i][5] &= ~Ausgabe_temp_unten;
			if (setzen == 2) Display_Shadow[x+i][5] = Ausgabe_temp_unten ^ 0xFF;
		}
	}

	else if (y >= 42 && y <= 49)
	{
		int z = 41; // Zeilen Pixel Zhler
		
		for (int i = 0; i < 6; i++)
		{
			Ausgabe_temp_oben = Zeichen[Ausgabe - ' '][i];
			Ausgabe_temp_oben = Ausgabe_temp_oben << (y-z);
			if (setzen == 0) Display_Shadow[x+i][5] |= Ausgabe_temp_oben;
			if (setzen == 1) Display_Shadow[x+i][5] &= ~Ausgabe_temp_oben;
			if (setzen == 2) Display_Shadow[x+i][5] = Ausgabe_temp_oben ^ 0xFF;
		}
		for (int i = 0; i < 6; i++)
		{
			Ausgabe_temp_unten = Zeichen[Ausgabe - ' '][i];
			Ausgabe_temp_unten = Ausgabe_temp_unten >> (8 - (y-z));
			Display_Shadow[x+i][6] = Ausgabe_temp_unten;
		}
	}

	else if (y >= 50 && y <= 57)
	{
		int z = 49; // Zeilen Pixel Zhler
		
		for (int i = 0; i < 6; i++)
		{
			Ausgabe_temp_oben = Zeichen[Ausgabe - ' '][i];
			Ausgabe_temp_oben = Ausgabe_temp_oben << (y-z);
			if (setzen == 0) Display_Shadow[x+i][6] |= Ausgabe_temp_oben;
			if (setzen == 1) Display_Shadow[x+i][6] &= ~Ausgabe_temp_oben;
			if (setzen == 2) Display_Shadow[x+i][6] = Ausgabe_temp_oben ^ 0xFF;
		}
		for (int i = 0; i < 6; i++)
		{
			Ausgabe_temp_unten = Zeichen[Ausgabe - ' '][i];
			Ausgabe_temp_unten = Ausgabe_temp_unten >> (8 - (y-z));
			Display_Shadow[x+i][7] |= Ausgabe_temp_unten;
		}
	}

	else if (y > 57)
	{
		for (int i = 0; i <	6; i++)
		{
			Display_Shadow[x+i][7] |= Zeichen[Ausgabe - ' '][i];
		}
	}
}


void Display_Bitmap(const unsigned char Bildschirm[])
{
	int Bitmap_Zeiger = 0;
	
	for (int Zeile=0; Zeile<8; Zeile++)								// 8 Zeilen vertikal.
	{
		for (int Spalte=0; Spalte<128; Spalte++)					// 16 Zeichen fllen, sind 128 Zeichen horizontal.
		{
			Display_Shadow[Spalte][Zeile] = pgm_read_byte(&Bildschirm[Bitmap_Zeiger]);
			Bitmap_Zeiger++;
		}
	}
}

void Display_Bitmap_XY(int x, int y, const unsigned char Graphik[], int Art)
{
	uint8_t Low_Byte, High_Byte;
	uint16_t Graphik_Breite;
	uint16_t Graphik_Hoehe;
	int Bitmap_Zeiger = 4;

	// Graphik Breite aus dem Bild extrahieren
	High_Byte = pgm_read_byte(&Graphik[0]); // Aus der Graphik die ersten beiden
	Low_Byte  = pgm_read_byte(&Graphik[1]); // Bytes extrahieren.

	Graphik_Breite = (High_Byte << 8) + Low_Byte;	// Die ersten acht Bits in eine 16 stellige kopieren,
												// um 8 stellen nach links, also ganz an den Anfang schieben
												// und die zweiten 8 Bits ans Ende kopieren.
	// Graphik Hoehe aus dem Bild extrahieren
	High_Byte = pgm_read_byte(&Graphik[2]);
	Low_Byte  = pgm_read_byte(&Graphik[3]);

	Graphik_Hoehe = (High_Byte << 8) + Low_Byte;

	for (int Zeile = 0; Zeile < 2; Zeile++)								// 8 Zeilen vertikal.
	{
		for (int Spalte = 0; Spalte < Graphik_Breite; Spalte++)					// 16 Zeichen fllen, sind 128 Zeichen horizontal.
		{
			Display_Shadow[Spalte+x][Zeile+y] = pgm_read_byte(&Graphik[Bitmap_Zeiger]);
			Bitmap_Zeiger++;
		}
	}
}



void Menue_Joystick_Sensor(void)
{
	Display_Bitmap_XY(0,0, Graphik_Menu_Links, 0);
	Display_aktualisieren();
	
	Display_Zeichen_Pixel( 6,3,'J',0);
	Display_Zeichen_Pixel(12,3,'o',0);
	Display_Zeichen_Pixel(18,3,'y',0);
	Display_Zeichen_Pixel(24,3,'s',0);
	Display_Zeichen_Pixel(30,3,'t',0);
	Display_Zeichen_Pixel(36,3,'.',0);
	Display_Zeichen_Pixel(42,3,'S',0);
	Display_Zeichen_Pixel(48,3,'e',0);
	Display_Zeichen_Pixel(54,3,'n',0);
	Display_Zeichen_Pixel(60,3,'s',0);
	Display_Zeichen_Pixel(66,3,'o',0);
	Display_Zeichen_Pixel(72,3,'r',0);

	Display_aktualisieren();
}