//
//  M A G N E T O
// ===============
// Datei         : Encoder.c
// Funktion      : Rotary Encoder abfragen und auswerten.
//
// Die Funktionen zum Abfragen der Encoder wurden aus dem Artikel
// von Mikrocontroller.net, Autor Peter Dannegger kopiert.
//

#include <avr/io.h>
#include <string.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include "Encoder.h"


// EAs fr die Rotary Switch
#define PHASE_A						(1 << PC6)
#define PHASE_B						(1 << PC7)
#define ENCODER_PIN					PINC

#define TRUE						1
#define FALSE						0

volatile int8_t enc_delta;	// -128 ... 127
static int8_t last;

int32_t val = 0; // Wert fr den Drehencoder

unsigned char Schritte_Ende_Links = TRUE;
unsigned char Schritte_Mitte = FALSE;
unsigned char Schritte_Ende_Rechts = FALSE;

volatile int Drehgeber_Schritte = 0;
volatile uint8_t Drehgeber_Schritte_aktiv = FALSE;

char val_Str[12];
char val_Str_temp[12];

void Encoder_INIT( void )
{
	int8_t new, tmp;

	// init encoder
	tmp = ENCODER_PIN;
	new = 0;
	if( tmp & PHASE_A ) new  = 3;
	if( tmp & PHASE_B ) new ^= 1;       // convert gray to binary
	last = new;                         // power on state
	enc_delta = 0;

	// init timer 0
	
	//TCCR0A = (1<<WGM01) | (1<<CS01) | (1<<CS00);   // CTC, prescaler 64
	TCCR0A = (1<<WGM01);
	TCCR0B |= ((1<<CS00) | (1<<CS01));   // CTC, prescaler 64
	OCR0A = (uint8_t)(F_CPU / 64.0 * 1e-3 - 0.5);   // 1ms
	TIMSK0 |= 1<<OCIE0A;
}


ISR( TIMER0_COMPA_vect ) {           // 1ms for manual movement
	int8_t new, diff, tmp;

	tmp = ENCODER_PIN;
	new = 0;
	if ( tmp & PHASE_B ) new  = 3;
	if ( tmp & PHASE_A ) new ^= 1;   // convert gray to binary
	diff = last - new;               // difference last - new
	if( diff & 1 ) {                 // bit 0 = value (1)
		last = new;                  // store new as next last
		enc_delta += (diff & 2) - 1; // bit 1 = direction (+/-)
	}
}


// Drehencoder einlesen
// read 1, 2, oder 4 step encoders
int8_t encode_read( uint8_t step )    {
	int8_t val;

	// atomic access to enc_delta
	cli();
	val = enc_delta;
	switch (step) {
		case  2: enc_delta = val & 1; val >>= 1; break;
		case  4: enc_delta = val & 3; val >>= 2; break;
		default: enc_delta = 0; break;
	}
	sei();
	return val;                   // counts since last call
}

void Encoder_Schritte_auswerten(void)
{
	val += encode_read(4);					// Encoder einlesen (mit 1 kHz abgetastet)

	if (val <= 0)							// Wenn der Encoder 0 oder kleiner ausgibt (wenn man weiter
	{										// nach links dreht, dann soll der Wert immer 0 sein.
		strcpy(val_Str, "001");
		val = 0;
		
		if (Schritte_Ende_Links == TRUE)	// Wenn das erste Mal der Wert 0 erreicht hat,
		{									// dann den Schirm mit den Pfeilsymbolen neu zeichnen.
			//Display_Shadow_loeschen();
			//Display_Bitmap(Bildschirm_04);
			Display_Bitmap_XY(91,0, Graphik_Inhalt_Leer, 0);
			Display_Bitmap_XY(91,0, Graphik_Inhalt_Rechts, 0);
			Display_aktualisieren();
			Schritte_Ende_Links = FALSE;
			Schritte_Mitte = TRUE;
		}
	}
	
	if (val == 1)
	{
		strcpy(val_Str, "002");
		
		if (Schritte_Mitte == TRUE)
		{
			//Display_Shadow_loeschen();
			//Display_Bitmap(Bildschirm_03);
			Display_Bitmap_XY(91,0, Graphik_Inhalt_Leer, 0);
			Display_Bitmap_XY(91,0, Graphik_Inhalt_Mitte, 0);
			Display_aktualisieren();
			Schritte_Ende_Links = TRUE;
			Schritte_Mitte = FALSE;
			Schritte_Ende_Rechts = TRUE;
		}
	}
	
	if (val == 2) strcpy(val_Str, "004");			// von 4 bis 64 knnen sich die Pfeile nicht ndern,
	if (val == 3) strcpy(val_Str, "008");			// da wir immer nach links und rechts drehen knnen.
	if (val == 4) strcpy(val_Str, "016");			// Somit braucht hier der Schirm nicht neu gezeichnet
	if (val == 5) strcpy(val_Str, "032");			// werden.
	if (val == 6) strcpy(val_Str, "064");

	if (val == 7)
	{
		strcpy(val_Str, "128");
		if (Schritte_Mitte == TRUE)
		{
			//Display_Shadow_loeschen();
			//Display_Bitmap(Bildschirm_03);
			Display_Bitmap_XY(91,0, Graphik_Inhalt_Leer, 0);
			Display_Bitmap_XY(91,0, Graphik_Inhalt_Mitte, 0);
			Display_aktualisieren();
			Schritte_Ende_Links = TRUE;
			Schritte_Mitte = FALSE;
			Schritte_Ende_Rechts = TRUE;
		}
	}
	
	if (val >= 8)
	{
		strcpy(val_Str, "256");
		val = 8;
		
		if (Schritte_Ende_Rechts == TRUE)
		{
			//Display_Shadow_loeschen();
			//Display_Bitmap(Bildschirm_05);
			Display_Bitmap_XY(91,0, Graphik_Inhalt_Links, 0);
			Display_aktualisieren();
			Schritte_Mitte = TRUE;
			Schritte_Ende_Rechts = FALSE;
		}
	}
	
	Display_Zeichen_Pixel(100, 3, val_Str_temp[0], 1);	// Die aktuellen Zahlen am Display lschen.
	Display_Zeichen_Pixel(106, 3, val_Str_temp[1], 1);
	Display_Zeichen_Pixel(112, 3, val_Str_temp[2], 1);
	
	Display_Zeichen_Pixel(100, 3, val_Str[0], 0);		// Die neuen Zahlen auf das Display schreiben.
	Display_Zeichen_Pixel(106, 3, val_Str[1], 0);
	Display_Zeichen_Pixel(112, 3, val_Str[2], 0);
	
	strcpy(val_Str_temp, val_Str);						// Die neuen Zahlen sichern, um sie beim Neuschreiben
														// vorher lschen zu knnen.
	
	Display_aktualisieren();
	
	
}

void Encoder_Schritte_abfragen(void)
{
	Encoder_Schritte_auswerten();
}

void Encoder_Menu_abfragen(void)
{
	
}