//
//  M A G N E T O
// ===============
// Datei         : Init.c
// Funktion      : Initialisiert die Hardware.
//

#include <avr/io.h>
#include <util/delay.h>
#include "Init.h"


uint8_t LEDs = 0b11111111; // Alle LEDs beim Start auf AUS setzen. (0 = EIN; 1 = AUS)

//
// DDR	(D)ata(D)irection(R)egister. Nach dem Reset sind alle Port Pins Eingnge. Mit '1' Ausgang setzen.
//
// PORT	Wenn DDR als Augang definiert ist, dann mit '1' Ausgang setzen und mit '0' Ausgang lschen.
//		Wenn DDR als Eingang definiert ist, dann mit '1' den internen Pullup aktiveren, mit '0' wieder entfernen.
//
// PIN	Wenn DDR als Eingang gesetzt ist, dann kann dieser abgefragt werden.
//
// Ausgang einschalten:	PORTn |= (1 << Bitnummer);	// BIT setzen.
// Ausgang ausschalten: PORTn &= ~(1 << Bitnummer);	// BIT lschen.
//
//
void Ports_INIT(void)
{
	// Alle PORTs sind nach dem RESET als Eingnge definiert.
	// Wir definieren nun die Ausgnge, alle anderen PORTs bleiben Eingnge.
	
	// Ausgangspins definieren
	
	DDRB = (1<<Display_A0) | (1<<Display_CS) | (1<<MOSI) | (1<<SCK) | (1<<T_165_CLK) | (1<<T_165_SHIFT);
	DDRD = (1 << Reset_Display) | (1<<RS485_DIR) | (1<<RCLK);
		
	// Nur vollstndigkeitshalber sind die Eingnge hier definiert. Beim Controllerstart sind die E/As
	// von Haus aus Eingnge.
	DDRA &= ~(1 << Taster_Joystick_SENSOR);					// Taster am Joystick SENSOR.
	DDRB &= ~(1 << T_165_DATEN);							// Daten Eingang von den Tastern als Eingang.
	PORTB &= ~(1 << T_165_DATEN);							// Daten Eingang von den Tastern Pullup deaktivieren.
	
	RCLK_LOW;												// Signal um vom Schieberegistern 74595 die bertragen Daten
															// auf die Ausgngen zu schalten (HI-aktiv)
}

void LEDs_INIT()
{
	SPI_74595_schreiben(LEDs);								// Achtung: Zuerst muss SPI initialisiert sein!
}

void Timer_LED_INIT(void)
{
	TCCR1A = 0;												// CTC-Mode und PRESCALE=1024
	TCCR1B = (1 << WGM12) | (1 << CS12) | (1 << CS10);
															// OutputCompare fr gewnschte Timer-Frequenz
	OCR1A =  7200;											//(unsigned short) ((unsigned long) F_CPU / PRESCALE / IRQS -1);
	
	TIMSK1 |= (1 << OCIE1A);								// OutputCompare-Interrupt A fr Timer 1
}


void RS232_0_INIT()
{
	UBRR0H = (UBRR_VAL_0 >> 8);								// Schnittstelle fr RS485 initialisieren.
	UBRR0L = UBRR_VAL_0;
	
	UCSR0B = (1<<RXEN0)|(1<<TXEN0);							// Receiver und Transmitter aktivieren.
	UCSR0C = (1<<UCSZ00) | (1<<UCSZ01);						// Format 8N1.
	UCSR0B |= (1 << RXCIE0);								//RX Interrupt aktivieren.
}

void SPI_MasterInit(void)
{
	SPCR =  (1 << SPE)|(1 << MSTR)|(1 << CPOL)|(1 << CPHA);
	SPSR = (1 << SPI2X);
}

void Display_INIT(void)
{
	PORTB &= ~(1<<Display_A0);								// Setze RST und A0 auf Low.
	PORTD &= ~(1 << Reset_Display);
	_delay_ms(50);											// Wartezeit laut Datenblatt fr die Initialisierung.

	PORTB |= (1 << Display_CS);								// Reset wieder zurck und ChipSelect auf High.
	PORTD |= (1 << Reset_Display);

	SPI_MasterInit();										// Initialisierungfolge laut Display Datenblatt.
	SPI_Display_schreiben(0x40);
	SPI_Display_schreiben(0xA1);
	SPI_Display_schreiben(0xC0);
	SPI_Display_schreiben(0xA6);
	SPI_Display_schreiben(0xA2);
	SPI_Display_schreiben(0x2F);
	SPI_Display_schreiben(0xF8);
	SPI_Display_schreiben(0x00);
	SPI_Display_schreiben(0x27);
	SPI_Display_schreiben(0x81);
	SPI_Display_schreiben(0x10);
	SPI_Display_schreiben(0xAC);
	SPI_Display_schreiben(0x00);
	SPI_Display_schreiben(0xAF);

	_delay_ms(5);
	PORTB &= ~(1 << Display_CS);
	
	Display_Shadow_loeschen();
	Display_aktualisieren();

	// Beim Start das Magneto Logo fr zwei Sekunden einblenden.
	//Display_Bitmap(Magneto_Logo);
	//Display_aktualisieren();
	//_delay_ms(2000);
	
	Display_Bitmap(Bildschirm_02);
	Display_aktualisieren();
	
	Menue_Joystick_Sensor();
}
