//
//  M A G N E T O
// ===============
// Datei         : Joystick.c
// Funktion      : Alle Routinen zur Joystick Abfrage und Auswertung.
//

#include <avr/io.h>
#include <string.h>
#include <stdio.h>
#include "Joystick.h"


int ADC_Wert;

char ADC_Wert_X_ASCII[5];
char ADC_Wert_X_ASCII_alt[5];

char ADC_Wert_Y_ASCII[5];
char ADC_Wert_Y_ASCII_alt[5];

char ADC_Wert_Z_ASCII[5];
char ADC_Wert_Z_ASCII_alt[5];

char ADC_Wert_Sensor_XV_ASCII[5];
char ADC_Wert_Sensor_XR_ASCII[5];

char ADC_Wert_Sensor_YV_ASCII[5];
char ADC_Wert_Sensor_YR_ASCII[5];

char ADC_Wert_Sensor_ZV_ASCII[5];
char ADC_Wert_Sensor_ZR_ASCII[5];

char ADC_Wert_Sensor_XV_ASCII_alt[5];
char ADC_Wert_Sensor_XR_ASCII_alt[5];

char ADC_Wert_Sensor_YV_ASCII_alt[5];
char ADC_Wert_Sensor_YR_ASCII_alt[5];

char ADC_Wert_Sensor_ZV_ASCII_alt[5];
char ADC_Wert_Sensor_ZR_ASCII_alt[5];

char ADC_Wert_Portal_XV_ASCII[5];
char ADC_Wert_Portal_XR_ASCII[5];

char ADC_Wert_Portal_YV_ASCII[5];
char ADC_Wert_Portal_YR_ASCII[5];

char ADC_Wert_Portal_ZV_ASCII[5];
char ADC_Wert_Portal_ZR_ASCII[5];

char ADC_Wert_Portal_XV_ASCII_alt[5];
char ADC_Wert_Portal_XR_ASCII_alt[5];

char ADC_Wert_Portal_YV_ASCII_alt[5];
char ADC_Wert_Portal_YR_ASCII_alt[5];

char ADC_Wert_Portal_ZV_ASCII_alt[5];
char ADC_Wert_Portal_ZR_ASCII_alt[5];

unsigned char Taster_Joystick_SENSOR_EIN = 0;

uint16_t X_Joy_Wert = 0;
uint16_t Y_Joy_Wert = 0;
uint16_t Z_Joy_Wert = 0;

void Joystick_INIT(void)
{
	//Versorgungsspannung als Referenz nuzten.
	ADMUX = (1 << REFS0);
	// Bit ADFR ("free running") in ADCSRA steht beim Einschalten
	// schon auf 0, also single conversion
	ADCSRA = (1<<ADPS0) | (1<<ADPS1) | (1<<ADPS2);     // Frequenzvorteiler (langsamste Wandlung whlen).
	ADCSRA |= (1<<ADEN);                  // ADC aktivieren

	// nach Aktivieren des ADC wird ein "Dummy-Readout" empfohlen, man liest
	// also einen Wert und verwirft diesen, um den ADC "warmlaufen zu lassen".
	ADCSRA |= (1<<ADSC);                  // eine ADC-Wandlung
	
	while (ADCSRA & (1<<ADSC))          // auf Abschluss der Konvertierung warten
	{
	}
	
	// ADCW muss einmal gelesen werden, sonst wird Ergebnis der nchsten
	// Wandlung nicht bernommen.
	(void) ADCW;
}


/* ADC Einzelmessung */
uint16_t ADC_Read( uint8_t channel )
{
	// Kanal waehlen, ohne andere Bits zu beeinfluen
	ADMUX = (ADMUX & ~(0x1F)) | (channel & 0x1F);
	ADCSRA |= (1<<ADSC);            // eine Wandlung "single conversion"
	while (ADCSRA & (1<<ADSC) ) {   // auf Abschluss der Konvertierung warten
	}
	return ADCW;                    // ADC auslesen und zurckgeben
}

uint16_t ADC_Read_Avg( uint8_t channel, uint8_t nsamples )
{
	uint32_t sum = 0;

	for (uint8_t i = 0; i < nsamples; ++i ) {
		sum += ADC_Read( channel );
	}

	return (uint16_t)( sum / nsamples );
}


int Skalieren(long in_min, long in_max, long out_min, long out_max, long Messwert)
{
	//	                                                       out_max - out_min
	//	  salierter_wert = out_min +  ( Messwert - in_min ) * ---------------------
	//	                                                        in_max - in_min
	//
	//	                                                       wenn du noch 1 Kommastelle haben willst, musst du noch einen Faktor 10
	//	                                                       anbringen
	//
	//				                                             10 * ( Messwert - in_min )  * ( out_max - out_min )
	//                          salierter_wert = 10 * out_min +  ----------------------------------------------------
	//                      	                                                       in_max - in_min
	//
	//	                                                       Auf Datentypen und mgliche berlufe achten!
	int Ergebnis;
	
	Ergebnis = (out_min + (Messwert - in_min)) * (out_max - out_min) / (in_max - in_min);
	
	return (Ergebnis);
}

void Joystick_Sensor_abfragen(void)
{
	strcpy(ADC_Wert_Portal_XV_ASCII, "0000");
	strcpy(ADC_Wert_Portal_XR_ASCII, "0000");
	strcpy(ADC_Wert_Portal_YV_ASCII, "0000");
	strcpy(ADC_Wert_Portal_YR_ASCII, "0000");
	strcpy(ADC_Wert_Portal_ZV_ASCII, "0000");
	strcpy(ADC_Wert_Portal_ZR_ASCII, "0000");
	
	// Joystick X-Achse abfragen
	ADC_Wert = ADC_Read_Avg(0,3);									// Kanal 0 = X-Achse, Anzahl Oversampling
	//ADC_Wert = ADC_Read(0);										// Zu Testzwecken ohne Oversampling. 
	
	if ((ADC_Wert > 440) && (ADC_Wert < 550))
	{
		X_Joy_Wert = 0;												// Hysterese, damit beim Bewegen vom Joystick in andere
		sprintf(ADC_Wert_Sensor_XV_ASCII, "%04i", X_Joy_Wert);		// Richtungen kein Einfluss auf diese Richtung besteht.
		sprintf(ADC_Wert_Sensor_XR_ASCII, "%04i", X_Joy_Wert);
	}																
	
	if (ADC_Wert < 440)
	{
		X_Joy_Wert = Skalieren(460,0,0,100,ADC_Wert);
		sprintf(ADC_Wert_Sensor_XV_ASCII, "%04i", X_Joy_Wert);
	}
	
	if (ADC_Wert > 550)
	{
		X_Joy_Wert = Skalieren(550,1023,0,100,ADC_Wert);
		sprintf(ADC_Wert_Sensor_XR_ASCII, "%04i", X_Joy_Wert);
	}
	sprintf(ADC_Wert_X_ASCII, "%04i", ADC_Wert);
	
	
	// Joystick Y-Achse abfragen.
	ADC_Wert = ADC_Read_Avg(1,3);									//Kanal 1 = Y-Achse, Anzahl Oversampling
	//ADC_Wert = ADC_Read(1);
	
	if ((ADC_Wert > 440) && (ADC_Wert < 550))
	{
		Y_Joy_Wert = 0;												// Hysterese, damit beim Bewegen vom Joystick in andere
		sprintf(ADC_Wert_Sensor_YV_ASCII, "%04i", Y_Joy_Wert);		// Richtungen kein Einfluss auf diese Richtung besteht.
		sprintf(ADC_Wert_Sensor_YR_ASCII, "%04i", Y_Joy_Wert);
	}

	if (ADC_Wert < 440)
	{
		Y_Joy_Wert = Skalieren(460,0,0,100,ADC_Wert);
		sprintf(ADC_Wert_Sensor_YV_ASCII, "%04i", Y_Joy_Wert);
	}

	if (ADC_Wert > 550)
	{
		Y_Joy_Wert = Skalieren(550,1023,0,100,ADC_Wert);
		sprintf(ADC_Wert_Sensor_YR_ASCII, "%04i", Y_Joy_Wert);
	}
	sprintf(ADC_Wert_Y_ASCII, "%04i", ADC_Wert);


	// Joystick Z-Achse abfragen.
	ADC_Wert = ADC_Read_Avg(2,3);									//Kanal 2 = Z-Achse, Anzahl Oversampling
	//ADC_Wert = ADC_Read(2);
	if ((ADC_Wert > 440) && (ADC_Wert < 550))
	{
		Z_Joy_Wert = 0;												// Hysterese, damit beim Bewegen vom Joystick in andere
		sprintf(ADC_Wert_Sensor_ZV_ASCII, "%04i", Z_Joy_Wert);		// Richtungen kein Einfluss auf diese Richtung besteht.
		sprintf(ADC_Wert_Sensor_ZR_ASCII, "%04i", Z_Joy_Wert);
	}

	if (ADC_Wert < 440)
	{
		Z_Joy_Wert = Skalieren(460,0,0,100,ADC_Wert);
		sprintf(ADC_Wert_Sensor_ZV_ASCII, "%04i", Z_Joy_Wert);
	}

	if (ADC_Wert > 550)
	{
		Z_Joy_Wert = Skalieren(550,1023,0,100,ADC_Wert);
		sprintf(ADC_Wert_Sensor_ZR_ASCII, "%04i", Z_Joy_Wert);
	}
	sprintf(ADC_Wert_Z_ASCII, "%04i", ADC_Wert);

	Joystick_Anzeige();
}

void Joystick_Portal_abfragen(void)
{
	
	
}

void Joystick_Anzeige(void)
{
	
	// Joystick Werte der X-Achse darstellen.
	Display_Zeichen_Pixel(22, 20, ADC_Wert_Sensor_XV_ASCII_alt[0], 1);
	Display_Zeichen_Pixel(28, 20, ADC_Wert_Sensor_XV_ASCII_alt[1], 1);
	Display_Zeichen_Pixel(34, 20, ADC_Wert_Sensor_XV_ASCII_alt[2], 1);
	Display_Zeichen_Pixel(40, 20, ADC_Wert_Sensor_XV_ASCII_alt[3], 1);
		
	Display_Zeichen_Pixel( 4, 20, 'X', 0);
	Display_Zeichen_Pixel(10, 20, 'V', 0);
	Display_Zeichen_Pixel(16, 20, '=', 0);
	Display_Zeichen_Pixel(22, 20, ADC_Wert_Sensor_XV_ASCII[0], 0);
	Display_Zeichen_Pixel(28, 20, ADC_Wert_Sensor_XV_ASCII[1], 0);
	Display_Zeichen_Pixel(34, 20, ADC_Wert_Sensor_XV_ASCII[2], 0);
	Display_Zeichen_Pixel(40, 20, ADC_Wert_Sensor_XV_ASCII[3], 0);
	
	strcpy(ADC_Wert_Sensor_XV_ASCII_alt, ADC_Wert_Sensor_XV_ASCII);

	Display_Zeichen_Pixel(68, 20, ADC_Wert_Sensor_XR_ASCII_alt[0], 1);
	Display_Zeichen_Pixel(74, 20, ADC_Wert_Sensor_XR_ASCII_alt[1], 1);
	Display_Zeichen_Pixel(80, 20, ADC_Wert_Sensor_XR_ASCII_alt[2], 1);
	Display_Zeichen_Pixel(86, 20, ADC_Wert_Sensor_XR_ASCII_alt[3], 1);

	Display_Zeichen_Pixel(50, 20, 'X', 0);
	Display_Zeichen_Pixel(56, 20, 'R', 0);
	Display_Zeichen_Pixel(62, 20, '=', 0);
	Display_Zeichen_Pixel(68, 20, ADC_Wert_Sensor_XR_ASCII[0], 0);
	Display_Zeichen_Pixel(74, 20, ADC_Wert_Sensor_XR_ASCII[1], 0);
	Display_Zeichen_Pixel(80, 20, ADC_Wert_Sensor_XR_ASCII[2], 0);
	Display_Zeichen_Pixel(86, 20, ADC_Wert_Sensor_XR_ASCII[3], 0);

	strcpy(ADC_Wert_Sensor_XR_ASCII_alt, ADC_Wert_Sensor_XR_ASCII);
	
	Display_Zeichen_Pixel(100, 20, ADC_Wert_X_ASCII_alt[0], 1);
	Display_Zeichen_Pixel(106, 20, ADC_Wert_X_ASCII_alt[1], 1);
	Display_Zeichen_Pixel(112, 20, ADC_Wert_X_ASCII_alt[2], 1);
	Display_Zeichen_Pixel(118, 20, ADC_Wert_X_ASCII_alt[3], 1);

	Display_Zeichen_Pixel(100, 20, ADC_Wert_X_ASCII[0], 0);
	Display_Zeichen_Pixel(106, 20, ADC_Wert_X_ASCII[1], 0);
	Display_Zeichen_Pixel(112, 20, ADC_Wert_X_ASCII[2], 0);
	Display_Zeichen_Pixel(118, 20, ADC_Wert_X_ASCII[3], 0);
	
	strcpy(ADC_Wert_X_ASCII_alt, ADC_Wert_X_ASCII);
	
	// Joystick Werte der Y-Achse darstellen.
	Display_Zeichen_Pixel(22, 35, ADC_Wert_Sensor_YV_ASCII_alt[0], 1);
	Display_Zeichen_Pixel(28, 35, ADC_Wert_Sensor_YV_ASCII_alt[1], 1);
	Display_Zeichen_Pixel(34, 35, ADC_Wert_Sensor_YV_ASCII_alt[2], 1);
	Display_Zeichen_Pixel(40, 35, ADC_Wert_Sensor_YV_ASCII_alt[3], 1);
	
	Display_Zeichen_Pixel( 4, 35, 'Y', 0);
	Display_Zeichen_Pixel(10, 35, 'V', 0);
	Display_Zeichen_Pixel(16, 35, '=', 0);
	Display_Zeichen_Pixel(22, 35, ADC_Wert_Sensor_YV_ASCII[0], 0);
	Display_Zeichen_Pixel(28, 35, ADC_Wert_Sensor_YV_ASCII[1], 0);
	Display_Zeichen_Pixel(34, 35, ADC_Wert_Sensor_YV_ASCII[2], 0);
	Display_Zeichen_Pixel(40, 35, ADC_Wert_Sensor_YV_ASCII[3], 0);
	
	strcpy(ADC_Wert_Sensor_YV_ASCII_alt, ADC_Wert_Sensor_YV_ASCII);

	Display_Zeichen_Pixel(68, 35, ADC_Wert_Sensor_YR_ASCII_alt[0], 1);
	Display_Zeichen_Pixel(74, 35, ADC_Wert_Sensor_YR_ASCII_alt[1], 1);
	Display_Zeichen_Pixel(80, 35, ADC_Wert_Sensor_YR_ASCII_alt[2], 1);
	Display_Zeichen_Pixel(86, 35, ADC_Wert_Sensor_YR_ASCII_alt[3], 1);

	Display_Zeichen_Pixel(50, 35, 'Y', 0);
	Display_Zeichen_Pixel(56, 35, 'R', 0);
	Display_Zeichen_Pixel(62, 35, '=', 0);
	Display_Zeichen_Pixel(68, 35, ADC_Wert_Sensor_YR_ASCII[0], 0);
	Display_Zeichen_Pixel(74, 35, ADC_Wert_Sensor_YR_ASCII[1], 0);
	Display_Zeichen_Pixel(80, 35, ADC_Wert_Sensor_YR_ASCII[2], 0);
	Display_Zeichen_Pixel(86, 35, ADC_Wert_Sensor_YR_ASCII[3], 0);

	strcpy(ADC_Wert_Sensor_YR_ASCII_alt, ADC_Wert_Sensor_YR_ASCII);
	
	Display_Zeichen_Pixel(100, 35, ADC_Wert_Y_ASCII_alt[0], 1);
	Display_Zeichen_Pixel(106, 35, ADC_Wert_Y_ASCII_alt[1], 1);
	Display_Zeichen_Pixel(112, 35, ADC_Wert_Y_ASCII_alt[2], 1);
	Display_Zeichen_Pixel(118, 35, ADC_Wert_Y_ASCII_alt[3], 1);

	Display_Zeichen_Pixel(100, 35, ADC_Wert_Y_ASCII[0], 0);
	Display_Zeichen_Pixel(106, 35, ADC_Wert_Y_ASCII[1], 0);
	Display_Zeichen_Pixel(112, 35, ADC_Wert_Y_ASCII[2], 0);
	Display_Zeichen_Pixel(118, 35, ADC_Wert_Y_ASCII[3], 0);
	
	strcpy(ADC_Wert_Y_ASCII_alt, ADC_Wert_Y_ASCII);
	
	
	// Joystick Werte der Z-Achse darstellen.
	Display_Zeichen_Pixel(22, 50, ADC_Wert_Sensor_ZV_ASCII_alt[0], 1);
	Display_Zeichen_Pixel(28, 50, ADC_Wert_Sensor_ZV_ASCII_alt[1], 1);
	Display_Zeichen_Pixel(34, 50, ADC_Wert_Sensor_ZV_ASCII_alt[2], 1);
	Display_Zeichen_Pixel(40, 50, ADC_Wert_Sensor_ZV_ASCII_alt[3], 1);
	
	Display_Zeichen_Pixel( 4, 50, 'Z', 0);
	Display_Zeichen_Pixel(10, 50, 'V', 0);
	Display_Zeichen_Pixel(16, 50, '=', 0);
	Display_Zeichen_Pixel(22, 50, ADC_Wert_Sensor_ZV_ASCII[0], 0);
	Display_Zeichen_Pixel(28, 50, ADC_Wert_Sensor_ZV_ASCII[1], 0);
	Display_Zeichen_Pixel(34, 50, ADC_Wert_Sensor_ZV_ASCII[2], 0);
	Display_Zeichen_Pixel(40, 50, ADC_Wert_Sensor_ZV_ASCII[3], 0);
	
	strcpy(ADC_Wert_Sensor_ZV_ASCII_alt, ADC_Wert_Sensor_ZV_ASCII);
	
	Display_Zeichen_Pixel(68, 50, ADC_Wert_Sensor_ZR_ASCII_alt[0], 1);
	Display_Zeichen_Pixel(74, 50, ADC_Wert_Sensor_ZR_ASCII_alt[1], 1);
	Display_Zeichen_Pixel(80, 50, ADC_Wert_Sensor_ZR_ASCII_alt[2], 1);
	Display_Zeichen_Pixel(86, 50, ADC_Wert_Sensor_ZR_ASCII_alt[3], 1);

	Display_Zeichen_Pixel(50, 50, 'Z', 0);
	Display_Zeichen_Pixel(56, 50, 'R', 0);
	Display_Zeichen_Pixel(62, 50, '=', 0);
	Display_Zeichen_Pixel(68, 50, ADC_Wert_Sensor_ZR_ASCII[0], 0);
	Display_Zeichen_Pixel(74, 50, ADC_Wert_Sensor_ZR_ASCII[1], 0);
	Display_Zeichen_Pixel(80, 50, ADC_Wert_Sensor_ZR_ASCII[2], 0);
	Display_Zeichen_Pixel(86, 50, ADC_Wert_Sensor_ZR_ASCII[3], 0);

	strcpy(ADC_Wert_Sensor_ZR_ASCII_alt, ADC_Wert_Sensor_ZR_ASCII);
	
	Display_Zeichen_Pixel(100, 50, ADC_Wert_Z_ASCII_alt[0], 1);
	Display_Zeichen_Pixel(106, 50, ADC_Wert_Z_ASCII_alt[1], 1);
	Display_Zeichen_Pixel(112, 50, ADC_Wert_Z_ASCII_alt[2], 1);
	Display_Zeichen_Pixel(118, 50, ADC_Wert_Z_ASCII_alt[3], 1);

	Display_Zeichen_Pixel(100, 50, ADC_Wert_Z_ASCII[0], 0);
	Display_Zeichen_Pixel(106, 50, ADC_Wert_Z_ASCII[1], 0);
	Display_Zeichen_Pixel(112, 50, ADC_Wert_Z_ASCII[2], 0);
	Display_Zeichen_Pixel(118, 50, ADC_Wert_Z_ASCII[3], 0);
	
	strcpy(ADC_Wert_Z_ASCII_alt, ADC_Wert_Z_ASCII);


	// Taster vom Joystick SENSOR abfragen und anzeigen

	if( PINA & ( 1 << Taster_Joystick_SENSOR ) )		// ist das Bit an dem der Taster hngt auf 1?
	{
		Taster_Joystick_SENSOR_EIN = 0;
		Display_Linie(  2, 13, 126, 0, 0);				// Zwei Striche horizontal oben.
		Display_Linie(  2, 14, 126, 0, 0);
		
		Display_Linie(  2, 13,  49, 1, 0);				// Zwei Striche vertikal links.
		Display_Linie(  3, 13,  49, 1, 0);
		
		Display_Linie(127, 13,  49, 1, 0);				// Zwei Striche vertikal rechts.
		Display_Linie(126, 13,  49, 1, 0);
		
		Display_Linie(  2, 62, 126, 0, 0);				// Zwei Striche horizontal unten.
		Display_Linie(  2, 63, 126, 0, 0);
	
	}

	else
	{
		Taster_Joystick_SENSOR_EIN = 1;
		Display_Linie(  2, 13, 126, 0, 1);
		Display_Linie(  2, 14, 126, 0, 1);
		
		Display_Linie(  2, 13,  49, 1, 1);
		Display_Linie(  3, 13,  49, 1, 1);
		
		Display_Linie(127, 13,  49, 1, 1);
		Display_Linie(126, 13,  49, 1, 1);
		
		Display_Linie(  2, 62, 126, 0, 1);
		Display_Linie(  2, 63, 126, 0, 1);	
	}

	
	Display_aktualisieren();
	
}