//
//  M A G N E T O
// ===============
// Datei         : RS485.c
// Funktion      : Senden und Empfangen ber die Schnittstelle,
//				   Protokoll auswerten.
//

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdbool.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "RS485.h"


char RS485_Empfang_Buffer[255];				// Alle ankommenden Daten werden hier reingeschrieben.
uint8_t RS485_Empfang_Buffer_Position = 0;	// Die aktuelle Position, um das neue empfangene Byte abzulegen
uint8_t RS485_Empfang_neues_Zeichen = 0;	// Ist 1, wenn ein neues Zeichen empfangen wurde (Parser starten).
uint8_t RS485_Empfang_komplett = 0;
char RS485_Senden_Buffer[200];

uint8_t RS485_Adresse = 10;					// Fr den Joystick ist die Adresse fest, keine DIP-Switches auf der Platine.
char RS485_Adresse_String[3];				// Die Adresse wird im Protokoll als ASCII bentigt.


int Nachrichten_Laenge_empfangen = 0;		// Variablen fr die Protokoll Auswertung.
char Nachricht_Laenge_empfangenen_String[4];

int Nachricht_Laenge_ermittelt = 0;
char Nachricht_Laenge_ermittelt_String[4];

char *Nachricht_Startzeichen = "#:";
char Nachricht_Empfaenger[3];
char Nachricht_Sender[3];
char Nachricht_Befehl[Befehl_Laenge];
char Nachricht_Daten[Daten_Laenge];
char *Nachricht_Endzeichen = ":#";

char Nachricht_Rueckmeldung[255];

char Befehle[Befehl_Anzahl][Befehl_Laenge] = {"Joystick","Status"};
char Daten_Joystick[Daten_Anzahl][Daten_Laenge] = {"EIN", "AUS"};
char Daten_Status[Daten_Anzahl][Daten_Laenge] = {"AKTUELL"};

bool Nachricht_vorhanden = false;				// Wenn eine Nachricht bis Sender ok ist, dann gilt die Nachricht als vorhanden,
												// sonst wird sie unkommentiert verworfen.
bool Nachricht_OK = false;
bool Nachricht_Befehl_erkannt = false;
bool Nachricht_Daten_erkannt = false;
bool Nachricht_Fehler_Trennzeichen = false;
bool Nachricht_Fehler_Laenge = false;
bool Nachricht_Fehler_Inhalt = false;
bool Nachricht_Fehler_keine_Daten = false;
bool Nachricht_Fehler_unbekannter_Befehl = false;
bool Nachricht_Fehler_unbekannte_Daten = false;
bool Nachricht_Befehle_vorhanden = true;

uint8_t i,j;									// temporre Schleifenvariablen.

extern char ADC_Wert_Sensor_XV_ASCII[5];
extern char ADC_Wert_Sensor_XR_ASCII[5];

extern char ADC_Wert_Sensor_YV_ASCII[5];
extern char ADC_Wert_Sensor_YR_ASCII[5];

extern char ADC_Wert_Sensor_ZV_ASCII[5];
extern char ADC_Wert_Sensor_ZR_ASCII[5];

extern char ADC_Wert_Portal_XV_ASCII[5];
extern char ADC_Wert_Portal_XR_ASCII[5];

extern char ADC_Wert_Portal_YV_ASCII[5];
extern char ADC_Wert_Portal_YR_ASCII[5];

extern char ADC_Wert_Portal_ZV_ASCII[5];
extern char ADC_Wert_Portal_ZR_ASCII[5];

extern bool Taste_freie_Fahrt;
extern bool Taste_Programmieren;
extern bool Taste_Automatik;
extern bool Taste_Stoerung;
extern bool Taste_Freigabe_Links;
extern bool Taste_Freigabe_Rechts;

extern char val_Str[12];

// -------------------------------------------------------------------------------------------------
// DIP-Switch einlesen und daraus die Adresse fr den Motor auf dem RS485 Bus festlegen.
// -------------------------------------------------------------------------------------------------
void RS485_Adresse_festlegen(void)
{
	//RS485_Adresse = PINA & (1<<ADR0 | 1<<ADR1 | 1<<ADR2);
	//RS485_Adresse += Motoren_Adresse_Offset;
	itoa(RS485_Adresse, RS485_Adresse_String, 10);				// Keine DIP-Switches auf dieser Platine!
}


ISR(USART0_RX_vect)												// Zeichen von RS485 empfangen
{
	unsigned char usart_rx = UDR0;								// ankommendes Zeichen zur Weiterverarbeitung sichern.

	RS485_Empfang_Buffer[RS485_Empfang_Buffer_Position] = usart_rx;
	RS485_Empfang_Buffer_Position++;
	
	if (usart_rx == '\n')
	{
		RS485_Empfang_Buffer_Position = 0;
		RS485_Empfang_komplett = 1;
		RS485_auf_Senden_schalten();							// Schnittstelle gleich auf Senden umschalten, damit
																// whrend der Verarbeitung kein neues Zeichen mehr angenommen wird.
	}

/*
	// Zeichen nur speichern, wenn es ein gltiges ASCII Zeichen von 32 bis 126 ist
	// und keine Auswertung des empfangenen Strings aktuell stattfindet.
	if (usart_rx >=32 && usart_rx <= 126 && RS485_Empfang_komplett == 0)
	{
		RS485_Empfang_Buffer[RS485_Empfang_Buffer_Position]=usart_rx;
		RS485_Empfang_Buffer_Position++;
	}
	else if (usart_rx != '\n')
		{
			RS485_Empfang_Buffer[RS485_Empfang_Buffer_Position] = '@';
			RS485_Empfang_Buffer_Position++;
			RS485_Empfang_Buffer_Position = 0;
			RS485_Empfang_komplett = 1;
			RS485_auf_Senden_schalten(); // Schnittstelle gleich auf Senden umschalten.
			// Whrend der Verarbeitung wird dann kein neues Zeichen mehr angenommen.
		}
	
	// Wenn ein LineFeed empfangen wurde, dann ist der String fertig zur Verarbeitung.
	if (usart_rx == '\n')
	{
		RS485_Empfang_Buffer[RS485_Empfang_Buffer_Position+1]= '\n';
		RS485_Empfang_Buffer[RS485_Empfang_Buffer_Position+2]= '\0';
		RS485_Empfang_Buffer_Position = 0;
		RS485_Empfang_komplett = 1;
		RS485_auf_Senden_schalten(); // Schnittstelle gleich auf Senden umschalten.
									 // Whrend der Verarbeitung wird dann kein neues Zeichen mehr angenommen. 
	}
	*/
	/*
	else
	{
		RS485_auf_Senden_schalten();
		RS485_Empfang_Buffer[RS485_Empfang_Buffer_Position+1]= '\0';
		uart0_puts(RS485_Empfang_Buffer);
		RS485_auf_Empfang_schalten();
		//RS485_Empfang_Buffer_Position = 0;
	}
	*/
	
}

void RS485_auf_Empfang_schalten(void)
{
	RS485_lesen;
}

void RS485_auf_Senden_schalten(void)
{
	RS485_schreiben;
	_delay_ms(4);							// Der MAX485 braucht laut Datenblatt 4ms beim Umschalten auf Senden.
}

int uart0_putc(unsigned char Zeichen)
{
	while (!(UCSR0A & (1<<UDRE0)))			// warten bis Senden mglich ist.
	{
	}

	UDR0 = Zeichen;							// Zeichen senden.
	return 0;
}

void uart0_puts (char *String)
{
	while (*String)
	{   
		uart0_putc(*String);				// String bis zum '\0' rausschreiben.
		String++;
		_delay_ms(1);
	}
}

char uart0_getnew()
{
	while (!(UCSR0A & (1<<RXC0)))
	;
	return UDR0;
}

//
// Auswertung von empfangener Nachricht ber die RS485 Schnittstelle.
//
int Nachricht_Auswertung(void)
{
	// #:nn:nn:xxx=xxx:...:nnn:#\n
	// | |  |  |       |   |  | |
	// | |  |  |	   |   |  | > Als Abschluzeichen ein LF.
	// | |  |  |       |   |  > Abschluzeichen einer Nachricht.  
	// | |  |  |       |   > Die Anzahl der bertragenen Bytes in diesem Nachricht.
	// | |  |  |       > Weitere Parameter mit Daten (maximal 5 insgesamt). 
	// | |  |  > Die ersten xxx sind die Befehle, die zweiten xxx sind die Daten.
	// | |  > Der Absender der Nachricht.
	// | > Der Empfnger der Nachricht.
	// > Startzeichen einer Nachricht.
	

				
	// Beginnt die empfangene Nachricht mit "#:", dann ist es ein gltiger Nachrichten Anfang.
	//
	// #:nn:nn:xxx=xxx:...:nnn:#\n
	// ^^-------------------------
	
	// Das LF am Ende der empfangenen Nachricht abschneiden
	// und durch '\0' ersetzen.
	
	strtok(RS485_Empfang_Buffer, "\n");
	int len = strlen(RS485_Empfang_Buffer);
	RS485_Empfang_Buffer[len] = '\0';
	
	// Den Pointer 'Nachricht' auf den Anfang der empfangenen Nachricht
	// setzen. Damit analysieren wir den Inhalt der Nachricht.
	char *Nachricht = RS485_Empfang_Buffer;
	
	if (strncmp(Nachricht_Startzeichen, Nachricht, 2) == 0)
	{
		if (DEBUG_MODUS)
		{
			uart0_puts("\nDEBUG MODUS\n");
			uart0_puts("---------------\n");
			uart0_puts("Nachricht...... = ");
			uart0_puts(Nachricht);
			uart0_puts("\nKopf........... = ");
			uart0_puts(Nachricht_Startzeichen);
			uart0_puts("\n");
		}
	}
	else
	{
		// Wenn der Kopf der Nachricht nicht korrekt ist,
		// dann die Analyse abbrechen und Nachricht verwerfen.
		
		if (DEBUG_MODUS)
		{
			uart0_puts("\nFehler......... = Kopf nicht korrekt!\n");
			uart0_puts("Nachricht...... = verworfen\n");
		}
		
		Nachricht_OK = false;
		//Nachricht_Fehler_Inhalt == TRUE;
		
		return -1;
	}

	// Ist die Nachricht fr diesen Empfnger gedacht?
	//
	// #:nn:nn:xxx=xxx:...:nnn:#\n
	// --^^-----------------------

	Nachricht += 2; // 2 Stellen in der Nachricht weiterspringen.
	
	if(strncmp(RS485_Adresse_String, Nachricht, 2) == 0)
	{
		strncpy(Nachricht_Empfaenger, Nachricht, 2); // Empfnger extrahieren
		strcpy(Nachricht_Empfaenger+2, "\0"); // und mit einem Null Byte abschlieen.
						
		if (DEBUG_MODUS)
		{
			uart0_puts("Empfnger...... = ");
			uart0_puts(Nachricht_Empfaenger);
			uart0_puts("\n");
		}
	}
	else
	{
		// Wenn der Empfnger nicht mit der eingestellten Adresse
		// bereinstimmt, die Analyse abbrechen und die Nachricht verwerfen.
		
		
		if (DEBUG_MODUS)
		{
			uart0_puts("\nEmpfnger...... = Das bin nicht ich!\n");
			uart0_puts("Nachricht...... = verworfen\n");
		}
		
		Nachricht_OK = false;
		//Nachricht_Fehler_Inhalt == TRUE;
		return -1;
	}

		
	// Korrektes Trennzeichen an dieser Stelle? Dann weiter in der Nachricht
	// und die Adresse vom Sender extrahieren.
	//
	// #:nn:nn:xxx=xxx:...:nnn:#\n
	// ----^^^--------------------
	Nachricht += 2;
		
	if(strncmp(":", Nachricht, 1) == 0) 
	{
		Nachricht += 1;
		strncpy(Nachricht_Sender, Nachricht, 2);
		strcpy(Nachricht_Sender+2, "\0");
		
		if (DEBUG_MODUS)
		{
			uart0_puts("Sender......... = ");
			uart0_puts(Nachricht_Sender);
			uart0_puts("\n");
		}
		
		Nachricht_vorhanden = true;
	}
	else
	{
		// Wenn an dieser Stelle kein Trennzeichen kommt,
		// die Analyse abbrechen und die Nachricht verwerfen.
		
		if (DEBUG_MODUS)
		{
			uart0_puts("\nFehler......... = Trennzeichen!\n");
			uart0_puts("Nachricht...... = verworfen\n");
		}
		
		Nachricht_OK = false;
		//Nachricht_Fehler_Inhalt == TRUE;

		strcpy(Nachricht_Rueckmeldung, "Trennzeichen-Fehler");
		return -1;
	}
			
	// Korrektes Trennzeichen an dieser Stelle? Dann weiter in der Nachricht
	// und die Lnge vom empfangen String messen und mit dem der angegebenen
	// Lnge in der Nachricht vergleichen.
	//
	// #:nn:nn:xxx=xxx:...:nnn:#\n
	// -------------------^^^^----
	Nachricht += 2;

	// Die Lnge der empfangenen Nachricht ermitteln.				
	Nachricht_Laenge_ermittelt = strlen(RS485_Empfang_Buffer);

	// Die Nachricht muss mehr als 13 Zeichen enthalten,
	// sonst knnen keine Daten mitgesendet worden sein.
	if (Nachricht_Laenge_ermittelt <= 13)
	{
		if (DEBUG_MODUS)
		{
			uart0_puts("\nFehler......... = Keine Nutzdaten\n");
			uart0_puts("Nachricht...... = verworfen\n");
		}
		
		Nachricht_OK = false;
		//Nachricht_Fehler_keine_Daten == TRUE;
		
		strcpy (Nachricht_Rueckmeldung, "Keine Nutzdaten");
		return -1;
	}
	
	itoa(Nachricht_Laenge_ermittelt, Nachricht_Laenge_ermittelt_String,10);
	sprintf(Nachricht_Laenge_ermittelt_String, "%03i", Nachricht_Laenge_ermittelt);

	if (DEBUG_MODUS)
	{
		uart0_puts("Lnge ermittelt = ");
		uart0_puts(Nachricht_Laenge_ermittelt_String);
		uart0_puts("\n");
	}
	
	// Wir springen zu den letzten fnf Zeichen der Nachricht.
	// Das letzte ist ein '\n' und wird ignoriert.
	// Die drei davor enthalten die Anzahl der bertragenen Bytes.
	// Diese vergleichen wir mit den ermittelten Bytes der empfangenen Nachricht.
	Nachricht = RS485_Empfang_Buffer + strlen(RS485_Empfang_Buffer) - 6;
	
	if (DEBUG_MODUS)
	{
		uart0_puts("Lnge Pointer.. = ");
		uart0_puts(Nachricht);
		uart0_puts("\nEmpfang Buffer. = ");
		uart0_puts(RS485_Empfang_Buffer);
		uart0_puts(" \n");	
					
	}
		
	if(strncmp(":", Nachricht, 1) == 0)
	{
		Nachricht += 1;
		strncpy(Nachricht_Laenge_empfangenen_String, Nachricht, 3);
		strcpy(Nachricht_Laenge_empfangenen_String+3, "\0");
		
		if (DEBUG_MODUS)
		{
			uart0_puts("Lnge empfangen = ");
			uart0_puts(Nachricht_Laenge_empfangenen_String);
			uart0_puts("\n");
		}
		
		if(strcmp(Nachricht_Laenge_ermittelt_String, Nachricht_Laenge_empfangenen_String) ==0)
		{
			//Wenn bei ermittelte Stringlnge gleich der empfangenen Info ist, dann weiter.
			if (DEBUG_MODUS)
			{
			uart0_puts("Lngen Check... = OK\n");	
			}
			Nachricht_Fehler_Laenge = false;
		}
		else
		{
			if (DEBUG_MODUS)
			{
				uart0_puts("\nFehler......... = Nachrichten Lnge inkorrekt!\n");
				uart0_puts("Nachricht...... = verworfen\n");
			}
			
			Nachricht_OK = false;
			Nachricht_Fehler_Laenge = true;
			strcpy(Nachricht_Rueckmeldung, "Falsche Nachrichtenlaenge");
			
			return -1;
		}
	}
	else
	{
		if (DEBUG_MODUS)
		{
			uart0_puts("\nFehler......... = Trennzeichen vor Nachrichtenlnge!\n");
			uart0_puts("Nachricht...... = verworfen\n");
		}
		Nachricht_OK = false;
		Nachricht_Fehler_Trennzeichen = true;
	
		strcpy(Nachricht_Rueckmeldung, "Trennzeichen-Fehler bei Lngenermittlung | ");
		strcat(Nachricht_Rueckmeldung, RS485_Empfang_Buffer);
		return -1;	
	}
				
	// Jetzt die Befehle mit den Daten extrahieren.
	// Zuerst den Zeiger vor das erste Zeichen des ersten Befehls setzen.
	// Korrektes Trennzeichen an dieser Stelle? Dann weiter zur Analyse.
	// (#:nn:nn:) = Dieses beginnt immer an 8. Stelle.
	// (12345678)
	
	//
	// #:nn:nn:xxx=xxx:...:nnn:#\n
	// --------^^^^^^^------------
	
	Nachricht = RS485_Empfang_Buffer + 7; // Bis zum Trennzeichen des Befehls vorspringen.
		
	if(strncmp(":", Nachricht, 1) == 0)
		{
			Nachricht ++;
			i=0;
		}
		else
		{

		if (DEBUG_MODUS)
		{
			uart0_puts("\nFehler......... = Trennzeichen!\n");
			uart0_puts("Nachricht...... = verworfen\n");
		}
		
			Nachricht_OK = false;
			Nachricht_Fehler_Trennzeichen = true;
			strcpy(Nachricht_Rueckmeldung, "Trennzeichen-Fehler");
			return -1;
		}

		// Ersten Befehl einlesen, bis ein '=' oder ein ':' kommt.
		// Sollte ein ':' errreicht werden, ist das ein Fehler.
		// Nur "Befehl=Parameter" ist gltig.
		
		while ( (strncmp(Nachricht, "=", 1) != 0) && (strncmp(Nachricht, ":", 1) != 0))
		{
			Nachricht_Befehl[i] = *Nachricht;	// Zeichen fr Zeichen kopieren,
			Nachricht_Befehl[i+1] = 0;			// und mit NULL abschlieen.
			i++;
			Nachricht ++;
		}
		
		Nachricht ++; // Das Trennzeichen "=" zu den Daten berspringen.
		
		if (DEBUG_MODUS)
		{
			uart0_puts("Befehl ........ = ");
			uart0_puts(Nachricht_Befehl);
			uart0_puts("\n");
		}
		
		// Ist der gesendete Befehl in der Liste der gltigen Befehle?
		for(int j=0; j < Befehl_Anzahl; j++)
		{
			if (strcmp(Befehle[j], Nachricht_Befehl) == 0)
			{
				Nachricht_Befehl_erkannt = true;
				break;
			}
		}

		if (Nachricht_Befehl_erkannt == true)
		{
			//uart0_puts("Befehl gefunden!\n");
			Nachricht_Befehl_erkannt = true; // Fehlererkennung wieder zurcksetzen.
		}
		else
		{
			if (DEBUG_MODUS)
			{
				uart0_puts("Befehl ........ = Nicht verstanden!\n");
				uart0_puts("Nachricht...... = verworfen\n");
			}
			
			Nachricht_OK = false;
			//Nachricht_Fehler_unbekannter_Befehl == TRUE;
			strcpy(Nachricht_Rueckmeldung, "Unbekannter Befehl [");
			strcat(Nachricht_Rueckmeldung, Nachricht_Befehl);
			strcat(Nachricht_Rueckmeldung, "]");
			return -1;
		}

		i = 0; // Zhler wieder verwenden.
	
		// Nach dem Befehl lesen wir nun die Daten dazu ein.
		// Sobald das nchste Trennzeichen ":" erreicht wird, haben wir die Daten.
		
		while ( (strncmp(Nachricht, ":", 1) != 0))
		{
			Nachricht_Daten[i] = *Nachricht;	// Zeichen fr Zeichen kopieren,
			Nachricht_Daten[i+1] = 0;			// und mit NULL abschlieen.
			i++;
			Nachricht ++;				
		}

		if (DEBUG_MODUS)
		{
			uart0_puts("Daten ......... = ");
			uart0_puts(Nachricht_Daten);
			uart0_puts("\n");
		}
		
		// Ist der gesendete Parameter in der Liste der gltigen Parameter?
		// Je nach Befehl nun die jeweiligen Daten dazu berprfen.
		
		
		//
		// Beim Befehl "Schritte_aktuell" die Daten einfach bernehmen.
		// Die berprfung auf Gltigkeit findet in der Visu statt.
		// Je nach Mikroschritte werden in der Visu auch die aktuellen Schritte
		// neu berechnet und hiermit der Motorsteuerung bergeben.
		//
		if (strcmp(Nachricht_Befehl, "Schritte_aktuell") == 0)
		{
			Nachricht ++;
		}
		
		//
		// Die gltige Daten zum Befehl "Joystick" berprfen.
		//
		if (strcmp(Nachricht_Befehl, "Joystick") == 0)
		{
			for(int j=0; j < Daten_Anzahl; j++)
			{
				if (strcmp(Daten_Joystick[j], Nachricht_Daten) == 0)
				{
					Nachricht_Daten_erkannt = true;
					break;
				}
			}
		

			if (Nachricht_Daten_erkannt == true)
			{
				Nachricht_Daten_erkannt = false; // Fehlererkennung wieder zurcksetzen.
				Nachricht ++;
			}

			else
			{
				Nachricht_OK = false;
				//Nachricht_Fehler_unbekannte_Daten == TRUE;
				strcpy(Nachricht_Rueckmeldung, "Unbekannte Daten zum Befehl \"Joystick\"");
				return -1;
			}
		}


		//
		// Die gltige Daten zum Befehl "Status" berprfen.
		//
		if (strcmp(Nachricht_Befehl, "Status") == 0)
		{
			for(int j=0; j < Daten_Anzahl; j++)
			{
				if (strcmp(Daten_Status[j], Nachricht_Daten) == 0)
				{
					Nachricht_Daten_erkannt = true;
					break;
				}
			}

			if (Nachricht_Daten_erkannt == true)
			{
				Nachricht_Daten_erkannt = true; // Fehlererkennung wieder zurcksetzen.
				Nachricht ++;
			}
			else
			{
				if (DEBUG_MODUS)
				{
					uart0_puts("Daten ......... = Nicht verstanden!\n");
					uart0_puts("Nachricht...... = verworfen\n");
				}
				
				Nachricht_OK = false;
				//Nachricht_Fehler_unbekannte_Daten == TRUE;
				strcpy(Nachricht_Rueckmeldung, "Unbekannte Daten zum Befehl \"Status\"");
				return -1;
			}
		}
		

		//
		// Das Ende der Nachricht auf das Endzeichen berprfen
		//
		if (strncmp(Nachricht_Endzeichen, Nachricht+3, 2) == 0)
		{
			// uart0_puts("Nachricht...... = verstanden\n");
		}
		else
		{
			if (DEBUG_MODUS)
			{
				uart0_puts("Fehler......... = Kein Endzeichen erkannt!");
				uart0_puts("Pointer........ = ");
				uart0_puts(Nachricht);
				uart0_puts("\n");
			}
			
			i=0;
			strcpy (Nachricht_Rueckmeldung, "Kein Endzeichen erkannt.");
			Nachricht_OK = false;
			return -1;
		}	
	
	Nachricht_OK = true;
	return 0;
} // Nachricht_Auswertung()

int Befehl_ausfuehren(void)
{
	//
	// Aktueller Status wurde angefordert.
	//
	// Rckantwort:
	// ============
	// SXV=nnn	(Joystick Sensor X Richtung vorwrts in 0 - 100%)
	// SXR=nnn	(Joystick Sensor X Richtung rckwrts in 0 - 100%)
	// SYV=nnn	(Joystick Sensor Y Richtung vorwrts in 0 - 100%)
	// SYR=nnn	(Joystick Sensor Y Richtung rckwrts in 0 - 100%)
	// SZV=nnn	(Joystick Sensor Z Richtung vorwrts in 0 - 100%)
	// SZR=nnn	(Joystick Sensor Z Richtung rckwrts in 0 - 100%)
	// PXV=nnn	(Joystick Portal X Richtung vorwrts in 0 - 100%)
	// PXR=nnn	(Joystick Portal X Richtung rckwrts in 0 - 100%)
	// PYV=nnn	(Joystick Portal Y Richtung vorwrts in 0 - 100%)
	// PYR=nnn	(Joystick Portal Y Richtung rckwrts in 0 - 100%)
	// PZV=nnn	(Joystick Portal Z Richtung vorwrts in 0 - 100%)
	// PZR=nnn	(Joystick Portal Z Richtung rckwrts in 0 - 100%)
	// TF=n		(Taster 'freie Fahrt' 0 (aus) oder 1 (ein)
	// TP=n		(Taster 'Programmieren' 0 (aus) oder 1 (ein)
	// TA=n		(Taster 'Automatik' 0 (aus) oder 1 (ein)
	// TS=n		(Taster 'Programmieren' 0 (aus) oder 1 (ein)
	// S=nnn	(Schritte Auswahl von 1 bis 256 in Zweierpotenz Schritten)

	if ( (strcmp(Nachricht_Befehl, "Status") == 0) && (strcmp(Nachricht_Daten, "AKTUELL") == 0))
	{
		strcpy (Nachricht_Befehl, "SXV");
		strcpy (Nachricht_Daten, ADC_Wert_Sensor_XV_ASCII);
		strcat (Nachricht_Daten, ":");

		strcat (Nachricht_Daten, "SXR=");
		strcat (Nachricht_Daten, ADC_Wert_Sensor_XR_ASCII);
		strcat (Nachricht_Daten, ":");

		strcat (Nachricht_Daten, "SYV=");
		strcat (Nachricht_Daten, ADC_Wert_Sensor_YV_ASCII);
		strcat (Nachricht_Daten, ":");

		strcat (Nachricht_Daten, "SYR=");
		strcat (Nachricht_Daten, ADC_Wert_Sensor_YR_ASCII);
		strcat (Nachricht_Daten, ":");

		strcat (Nachricht_Daten, "SZV=");
		strcat (Nachricht_Daten, ADC_Wert_Sensor_ZV_ASCII);
		strcat (Nachricht_Daten, ":");

		strcat (Nachricht_Daten, "SZR=");
		strcat (Nachricht_Daten, ADC_Wert_Sensor_ZR_ASCII);
		strcat (Nachricht_Daten, ":");

		strcat (Nachricht_Daten, "PXV=");
		strcat (Nachricht_Daten, ADC_Wert_Portal_XV_ASCII);
		strcat (Nachricht_Daten, ":");
		
		strcat (Nachricht_Daten, "PXR=");
		strcat (Nachricht_Daten, ADC_Wert_Portal_XR_ASCII);
		strcat (Nachricht_Daten, ":");

		strcat (Nachricht_Daten, "PYV=");
		strcat (Nachricht_Daten, ADC_Wert_Portal_YV_ASCII);
		strcat (Nachricht_Daten, ":");

		strcat (Nachricht_Daten, "PYR=");
		strcat (Nachricht_Daten, ADC_Wert_Portal_YR_ASCII);
		strcat (Nachricht_Daten, ":");

		strcat (Nachricht_Daten, "PZV=");
		strcat (Nachricht_Daten, ADC_Wert_Portal_ZV_ASCII);
		strcat (Nachricht_Daten, ":");

		strcat (Nachricht_Daten, "PZR=");
		strcat (Nachricht_Daten, ADC_Wert_Portal_ZR_ASCII);
		strcat (Nachricht_Daten, ":");

		strcat (Nachricht_Daten, "TF=");
		if (Taste_freie_Fahrt == 0) strcat (Nachricht_Daten, "0");
		if (Taste_freie_Fahrt == 1) strcat (Nachricht_Daten, "1");
		strcat (Nachricht_Daten, ":");

		strcat (Nachricht_Daten, "TP=");
		if (Taste_Programmieren == 0) strcat (Nachricht_Daten, "0");
		if (Taste_Programmieren == 1) strcat (Nachricht_Daten, "1");
		strcat (Nachricht_Daten, ":");

		strcat (Nachricht_Daten, "TA=");
		if (Taste_Automatik == 0) strcat (Nachricht_Daten, "0");
		if (Taste_Automatik == 1) strcat (Nachricht_Daten, "1");
		strcat (Nachricht_Daten, ":");

		strcat (Nachricht_Daten, "TS=");
		if (Taste_Stoerung == 0) strcat (Nachricht_Daten, "0");
		if (Taste_Stoerung == 1) strcat (Nachricht_Daten, "1");
		strcat (Nachricht_Daten, ":");

		strcat (Nachricht_Daten, "S=");
		strcat (Nachricht_Daten, val_Str);

		return 0;
	}

	
	if ( (strcmp(Nachricht_Befehl, "Joystick") == 0) && (strcmp(Nachricht_Daten, "EIN") == 0))
	{
		//strcpy(Nachricht_Befehl, "Joystick");
		//strcpy(Nachricht_Daten, "EIN");
		LED_Status_EIN();
		return 0;
	}

	if ( (strcmp(Nachricht_Befehl, "Joystick") == 0) && (strcmp(Nachricht_Daten, "AUS") == 0))
	{
		//strcpy(Nachricht_Befehl, "Joystick");
		//strcpy(Nachricht_Daten, "AUS");
		LED_Status_AUS();
		return 0;
	}

	return 0;
} // Befehl ausfuehren()

	
